import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;

/**This has all the definitions for the floating point
   definitions of the heat semigroup*/

public class HeatSemigroup {

    /**This does one of the elements of the semigroup
            Here a[0] = 0,1,2,3,4
            a[1] = 0,1
            a[2] = 0,1,2,3,...  power of map
            a[3] = 0,1          when a[3]=1 we use an auxiliary map not in the semigroup.
    **/
    public static double[] main(int[] a,double[] x) {
	if(a[3]==1) return map(B2(x)); 
	return B1(map(a,B2(x))); 
    }

    /**Here is the projective heat map in flag coordinates.*/

    public static double[] map(int[] a,double[] x) {
	double[] y={x[0],x[1]};
	for(int i=0;i<a[2];++i) y=map(y);
	y=gauss(y,a[1]);
	double[] z={y[a[0]],y[1-a[0]]};
	return(z);
    }

    public static double[] map(double[] x) {
        double[] w={coord1(x[0],x[1]),coord1(x[1],x[0])};
	return(w);
    }

    public static double coord1(double x,double y) {
	double d=1;
	d=d*(-3+2*x*y+x*y*y);
	d=d*(+6-x-6*x*y+x*x*y*y);
	d=d/(-5+x-y+4*x*y+x*y*y);
	d=d/(+6-y-6*x*y+x*x*y*y);
	return(d);
    }

    /**Here is the Gauss recurrence.  It commutes with map()*/

    public static double[] gauss(double[] d) {
	double[] e={d[1],(1-d[0])/(1-d[0]*d[1])};
	return(e);
    }

    public static double[] gauss(double[] d,int q) {
	double[] e={d[0],d[1]};
	for(int i=0;i<q;++i) e=gauss(e);
	return(e);
    }

    /**This is the coordinatewise Mobius transformation B which
       conjugates the heat map in flag coordinates to the ones
       displyed in this program.  The map is B1(x,y)=(b1(x),b1(y),
       where b1 is the Mobius transformation with the following action:
       1/phi  --> infinity    (attracting fixed pt)
       -phi   --> 0           (symmetric repelling fixed point*)
       -3     --> 1           (asymmetric repelling fixed pt
       The map B2 is the inverse of B1.
    **/

    public static double[] B1(double[] x) {
	double[] y={b1(x[0]),b1(x[1])};
	return(y);
    }
    public static double[] B2(double[] x) {
	double[] y={b2(x[0]),b2(x[1])};
	return(y);
    }
    public static double b1(double x) {
	double phi=QI.phi();
	double a=-1-2*phi;
	double b=-2-3*phi;
	double c=-1*phi;
	double d=+1;
	double y=(a*x+b)/(c*x+d);
	return(y);
    }
    public static double b2(double x) {	
	double phi=QI.phi();
	double a=+11-7*phi;
	double b=+1-2*phi;
	double c=-7+4*phi;
	double d=+3-1*phi;
	double y=(a*x+b)/(c*x+d);
	return(y);
    }
}

