import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;





public class QI2Confiner implements Runnable {
    QI2RationalMap MAP;
    QI2Polygon DOMAIN;
    QI2Polygon CONTAINER;
    QITestWrapper QIT;
    int EDGE,TRI,TEST;
    long TIME;
    MonitorTests MT;
    int STRICT;

    /**This tests presupposes that the denominators of the rational map
       do not vanish in the interior of the given CONTAINER.  
       If the nonvanishing holds, and if this test succeeds, 
       it implies that X(DOMAIN) is contained in the domain bounded by CONTAINER. 
       Here DOMAIN and CONTAINER are polygons.  CONTAINER 
       need not be convex, but when CONTAINER is nonconvex, 
       the answer might not be right. The test checks that X(DOMAIN)
       lies on the correct side of all the half planes bounded
       by lines extending the edges of CONTAINER.  When CONTAINER 
       is convex, this intersection is precisely CONTAINER.  When
       CONTAINER is not convex, the intersection is smaller.  
       The routine runs twice sometimes. **/

    public QI2Confiner() {}

    public QI2Confiner(QITestWrapper qit,QI2RationalMap x,QI2Polygon u,QI2Polygon v,int strict) {
	MAP=x;
	DOMAIN=u;
	CONTAINER=v;
	QIT=qit;
	TIME=time();
	MT=QIT.M.TC.MT;
	STRICT=strict;
    }

    public void run() {
	sendConfineInit();
	boolean test=mainTest();
	if(test==false) {
           QIT.HALT=true;
	   MT.S[0]="test fail";
	   MT.STATE=2;
	}
	if(test==true) {
	   MT.S[0]="test pass";  
           MT.STATE=1;	
	}

	QIT.HALT=true;
	QIT.M.repaint();
    }


    public boolean mainTest() {
	int N=CONTAINER.COUNT;   //number of edges
	int sign0=0;
	int sign1=0;
	int i=0;
	while((i<N)&&(QIT.HALT==false)) {
	    EDGE=i;
	    sendState();
            int j=(i+1)%N; 
	    sign1=isConfined(MAP,DOMAIN,CONTAINER.V[i],CONTAINER.V[j]);
	    boolean check=signCheck(sign0,sign1);
	    if(check==false) {
	       QIT.HALT=true;
	       return(false);
	    }
	    sign0=sign1;
	    ++i; 
	}
	if(QIT.HALT==false) return(true);
	return(false);
    }

   public int isConfined(QI2RationalMap X,QI2Polygon W,QI2[] P,QI2[] Q) {
        QI2Polygon[] LIST=W.toSubdivision();

	System.out.println("length "+LIST.length);


	int test=0;
        for(int i=0;i<LIST.length;++i) {
	   TRI=i;
	   sendState();
	   test=numerTest(X,LIST[i],P,Q);
	   if(test==0) return(0);
       }
       return(test);
   }

    /**If this test succeeds, it implies that X(T) is disjoint from the line PQ.
       Here T is a triangle.*/
	       
    public int numerTest(QI2RationalMap X,QI2Polygon W,QI2[] P,QI2[] Q) {
	QI2Polynomial Y=X.confine(P,Q);
	int test=0;
	for(int i=0;i<3;++i) {
	    TEST=i+2;
	    sendState();
	    W=W.rotate();
	    test=vanishingTest(Y,W);
	    if(test==0) clearPoly();
	    if(test!=0) return(test);
	}
	return(0);
    }

    public int vanishingTest(QI2Polynomial Y,QI2Polygon T) {
	int count=0;
	int record=0;
	QI2Polygon[] LIST={T};
	int sign0=0;
	while((QIT.HALT==false)&&(LIST.length>0)) {
	    QI2Polygon W=LIST[LIST.length-1]; 
	    if(record<W.age) record=W.age; 
            sendDepth(W.age,record,count);
	    int sign1=checkDominance(Y,W); 
	    boolean test=signCheck(sign0,sign1);
	    if(record>7) return(0);
	    if(test==true) { 
                ++count;
		sign0=sign1;
                LIST=QI2Polygon.truncate(LIST);	
	    }
	    if(test==false)   LIST=QI2Polygon.refine(LIST);  
	}
	if(QIT.HALT==false) return(sign0);
	return(0);
    }


    public int checkDominance(QI2Polynomial Y,QI2Polygon WW) {
	QI2Polygon W=new QI2Polygon(WW);
	for(int i=0;i<3;++i) {
	    int sign=Y.checkDominance(W,STRICT);
	   if(sign!=0) {
              sendPoly(W);
              return(sign);
	   }
	   W=W.rotate();
	}
	return(0);
    }

    public boolean signCheck(int sign0,int sign1) {
        boolean test=true;
	if(sign1==0) test=false;
	if((sign0!=0)&&(sign1!=sign0)) test=false;
	return(test);
    }

    /**These are communications with the program*/

    public void sendConfineInit() {
	String S="confiner test  ";
	Integer E=new Integer(CONTAINER.COUNT);
	int c=DOMAIN.COUNT;
	Integer T=new Integer(c-2);
	S=S+E.toString() +" edges  ";
	S=S+T.toString()+" triangles";
	MT.S[0]=S;
    }

    public void sendPoly(QI2Polygon W) {
	Tracer TR=QIT.M.P.TR;
	int c=TR.COUNT;
	W.testnumber=EDGE;
	W.color=EDGE;
	TR.LIST[c]=W;
	++TR.COUNT;
    }
    public void clearPoly() {
	Tracer TR=QIT.M.P.TR;
	TR.COUNT=0;
    }

    public void sendState() {
	int e1=EDGE;
	int e2=(EDGE+1)%CONTAINER.COUNT;
	Integer I1=new Integer(e1+1);
	Integer J1=new Integer(e2+1);
	Integer I2=new Integer(TRI+1);
	Integer I3=new Integer(TEST-1);
        MT.S[1]="edge "+I1.toString()+"  triangle "+I2.toString()+"  try "+I3.toString();
    }

    public void sendDepth(int a,int b,int c) {
	MT.DEPTH[0]=a;
	MT.DEPTH[1]=b;
	MT.DEPTH[2]=c;
	MT.TIME=time()-TIME;
	QIT.M.repaint();
    }


    public static long time() {
	long t=System.currentTimeMillis();
	return(t);
    }


}







