import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

/**This is a pair of rational functions defined over Z[sqrt 5].*/

public class QI2RationalMap {
    QI2RationalFunction[] F=new QI2RationalFunction[2];

    public QI2RationalMap() {
	F[0]=new QI2RationalFunction();
	F[1]=new QI2RationalFunction();
    }

    /**copies a QIRationalMap*/

    public QI2RationalMap(long[][][] m) {
	F[0]=new QI2RationalFunction();
	F[1]=new QI2RationalFunction();
	F[1].P=new QI2Polynomial(m[0],m[4]);
	F[1].Q=new QI2Polynomial(m[2],m[6]);
	F[0].P=new QI2Polynomial(m[1],m[5]);
	F[0].Q=new QI2Polynomial(m[3],m[7]);
    }

    public QI2RationalMap reverse() {
	QI2RationalMap X=new QI2RationalMap();
	X.F[0].P=new QI2Polynomial(this.F[1].P);
	X.F[0].Q=new QI2Polynomial(this.F[1].Q);
	X.F[1].P=new QI2Polynomial(this.F[0].P);
	X.F[1].Q=new QI2Polynomial(this.F[0].Q);
	return(X);
    }

    /**the identity map*/
    public static QI2RationalMap identityMap() {
	long[][][] m={
        {{0,1},{0,0}},
        {{0,0},{1,0}},
        {{1,0},{0,0}}, 
        {{1,0},{0,0}}, 
        {{0,0},{0,0}}, 
        {{0,0},{0,0}}, 
	{{0,0},{0,0}}, 
        {{0,0},{0,0}}};
	QI2RationalMap X=new QI2RationalMap(m);
	return(X);
    }

    /**The linear functional is Ax+By+C=0.*/

    public QI2RationalFunction composeWithFunctional(QI2 A,QI2 B,QI2 C) {
	QI2RationalFunction X0=this.F[0].scale(A);
	QI2RationalFunction X1=this.F[1].scale(B);
	QI2RationalFunction X2=new QI2RationalFunction(C);
	X2=QI2RationalFunction.plus(X2,X1);
	X2=QI2RationalFunction.plus(X2,X0);
	return(X2);
    }


    /**Let L be the line through the points P and Q.
       Let X be a rational map.  This produces the
       (L compose X).  Here
       L* is a linear functional that vanishes on L.**/

    public QI2Polynomial confine(QI2[] P,QI2[] Q) {
	QI2RationalFunction RY=confineRational(P,Q);
	return(RY.P);
    }

    public QI2RationalFunction confineRational(QI2[] P,QI2[] Q) {
	QI2[] L=getLinearFunctional(P,Q);
	QI2RationalFunction Y=composeWithFunctional(L[0],L[1],L[2]);
	return(Y);
    }

    public static QI2[] getLinearFunctional(QI2[] P,QI2[] Q) {
	QI2[] R=new QI2[2];
	R[0]=QI2.plus(P[0],QI2.minus(P[1],Q[1]));
	R[1]=QI2.plus(P[1],QI2.minus(Q[0],P[0]));
	QI2 A=QI2.minus(R[0],P[0]);
	QI2 B=QI2.minus(R[1],P[1]);
	QI2 C=QI2.dot(P,QI2.minus(P,R));
	QI2[] L={A,B,C};
	return(L);
    }

    public double[] evaluate(double x,double y) {
	double[] X={F[0].evaluate(x,y),F[1].evaluate(x,y)};
	return(X);
    }

    public void print() {
	System.out.println("start rational map");
	F[0].P.print();
	F[0].Q.print();
	System.out.println("------");
	F[1].P.print();
	F[1].Q.print();
	System.out.println("end rational map");
    }

    public QI2RationalMap fixSign() {
	QI2RationalMap X=new QI2RationalMap();
	X.F[0]=F[0].fixSign();
	X.F[1]=F[1].fixSign();
	return(X);
    }
}





