import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class QITestWrapper implements Runnable {
    Manager M;
    boolean HALT;
    QI2PositiveTest QI2P;
    QI2Confiner QI2C;
    QI2Excluder QI2E;
    MonitorTests MT;

    public QITestWrapper() {
	HALT=true;
    }

    public QITestWrapper(Manager MM) {
	this.M=MM;
	HALT=true;
	QI2P=new QI2PositiveTest();
	QI2C=new QI2Confiner();
	QI2E=new QI2Excluder();
	M.TC.FAIL_MESSAGE[0]="";
	M.TC.FAIL_MESSAGE[1]="";
    }


   public void run() {
       boolean test=checkBad();
       if(test==true) return;
       HALT=false;
       M.TC.FAIL=false;	
       M.TC.MT=new MonitorTests();
       MT=M.TC.MT;
       int piece = SelectPartition.TILE;
	if(M.TC.TEST.mode==0) runDenominator(piece);
	if(M.TC.TEST.mode==1) runConfiner(piece);
	if(M.TC.TEST.mode==2) runExcluder(piece);
	if(M.TC.TEST.mode==3) runArea(piece);
	if(M.TC.TEST.mode==4) runNorm(piece);
	if(M.TC.TEST.mode==5) runDiff(piece,4);
	if(M.TC.TEST.mode==6) runDiff(piece,2);
	M.repaint();
	HALT=true;
    }

    public boolean checkBad() {
	if((M.TC.STRICT.mode==2)&&(M.TC.TEST.mode!=0)) {
	    M.TC.FAIL=true;
	    M.TC.FAIL_MESSAGE[0]="Test mode 2 only works";
	    M.TC.FAIL_MESSAGE[1]="for the denominator test";
	    return true;
	}
	return false;
    }


    public void runDenominator(int p) {
	QI2Polynomial.SIZE=16;	
        QI2RationalMap X=getMap();
	QI2Polygon DOMAIN=getDomain(p);
	QI2Polynomial[] F={X.F[0].Q,X.F[1].Q};
	int mode=M.TC.STRICT.mode;

	/**In case we do the weak test, we have the
           option of subtracting off
           a small function that is non-negative in the
           interior of the fundamental domain. This allows 
           to conclude that the relevant functions are
           positive in the interior of the fundamental domain
           and not just non-negative*/

	if(mode==2) {
	    F[0]=plus(F[0],special());
	    F[1]=plus(F[1],special());
	    mode=0;
	}


	F[0]=F[0].normalize(DOMAIN.interiorPoint());
	F[1]=F[1].normalize(DOMAIN.interiorPoint());
	QI2P=new QI2PositiveTest(this,F,DOMAIN,mode);
        QI2P.run();
    }

    public void runConfiner(int p) {
	QI2Polynomial.SIZE=16;	
        QI2RationalMap X=getMap();	
        QI2Polygon DOMAIN=getDomain(p);
        QI2Polygon RANGE=M.P.RANGE;
	QI2Confiner QI2C=new QI2Confiner(this,X,DOMAIN,RANGE,M.TC.STRICT.mode);
        QI2C.run();
    }

    public void runExcluder(int p) {
	QI2Polynomial.SIZE=16;	
        QI2RationalMap X=getMap();	
        QI2Polygon DOMAIN=getDomain(p);
        QI2Polygon V=M.P.RANGE;
	QI2Excluder QI2E=new QI2Excluder(this,X,DOMAIN,V,M.TC.STRICT.mode);
        QI2E.run();
    }

    public void runArea(int p) {
	QI2 bound=M.TC.HTA.getBound();
	if(bound.sign()==-1) {
	    M.TC.FAIL=true;
	    M.TC.FAIL_MESSAGE[0]="fail: bound negative";
	    M.TC.FAIL_MESSAGE[1]="we only test det(dT)>=0";
	    return;
	}
	QI2Polynomial.SIZE=32;	
	QI2Polygon DOMAIN=getDomain(p);
	QI2[] POINT=DOMAIN.interiorPoint();
	QI2RationalMap X=getMap();
	QI2RationalFunction JX=QI2Differential.detD(X);
	JX=JX.makePositive(POINT);
	QI2RationalFunction JX2=minus(JX,bound);
	JX2.fixSign(POINT);
	boolean test=JX2.purePositive(POINT);
	if(test==false) {
	    M.TC.FAIL=true;
	    M.TC.FAIL_MESSAGE[0]="fail: first evaluation negative";
	    M.TC.FAIL_MESSAGE[1]="try a smaller bound";
	    return;
	}
	QI2Polynomial[] POLY={JX2.P};
	int mode=M.TC.STRICT.mode;
	QI2P=new QI2PositiveTest(this,POLY,DOMAIN,mode);
	System.out.println("area lower bound "+bound.toDouble());
        QI2P.run();
    }




    public void runNorm(int p) {
	QI2[] b=M.TC.HTN.getBounds();
	QI2Polynomial.SIZE=32;
	QI2Polygon DOMAIN=getDomain(p);	
	QI2RationalMap X=getMap();

	QI2RationalFunction S1=QI2Differential.sumOfSquares1D(X);
	QI2RationalFunction S2=QI2Differential.sumOfSquares2D(X);

	QI2RationalFunction B1=minus(S1,b[0]);
	QI2RationalFunction B2=minus(S2,b[1]);
	B1=B1.fixSign(DOMAIN.interiorPoint());
	B2=B2.fixSign(DOMAIN.interiorPoint());
	B1=B1.negate();
	B2=B2.negate();
	QI2Polynomial[] POLY={B1.P,B2.P};

	QI2P=new QI2PositiveTest(this,POLY,DOMAIN,1);
	System.out.println("square sum upper bound ");
	b[0].println();
	b[1].println();
	double t=Math.sqrt(b[0].toDouble()+b[1].toDouble());
	System.out.println("max expansion bound: "+t);
        QI2P.run();
    }


    /**choice=2 corresponds to the stretch test.
       choice=4 corresponde to the cone test.*/

    public void runDiff(int p,int choice) {
	QI2Polynomial.SIZE=32;	
	QI2Polygon DOMAIN=getDomain(p);
	QI2RationalMap X=getMap();
	QI2[][] b=M.TC.HTS.getBounds();
	if(choice==4)  b=M.TC.HTC.getBounds();
	QI2[] POINT=DOMAIN.interiorPoint();

	if(choice==2) System.out.println("stretch test ");
	if(choice==4) System.out.println("cone ");
	QI2Polynomial[] POLY=new QI2Polynomial[choice];
	for(int k=0;k<choice;++k) {
	    QI2RationalFunction[] DFV=QI2Differential.DIFF(X,b[k][0],b[k][1]);
	    QI2RationalFunction F=times(DFV[0],b[k][2]);
	    F=plus(F,times(DFV[1],b[k][3]));
	    F=minus(F,b[k][4]);
	    F=F.fixSign(POINT);

	    boolean test=F.purePositive(POINT);
	    if(test==false) {
	       M.TC.FAIL=true;
	       M.TC.FAIL_MESSAGE[0]="fail: first evaluation negative";
	       if(choice==2) 
               M.TC.FAIL_MESSAGE[1]="try switching the sign of B";
	       if(choice==4) 
               M.TC.FAIL_MESSAGE[1]="try negating the cone";
	       return;
	    }
	    POLY[k]=F.P;
	}
	QI2P=new QI2PositiveTest(this,POLY,DOMAIN,M.TC.STRICT.mode);  
        QI2P.run();
    }






    public QI2Polygon getDomain(int p) {
	return(M.P.DOMAIN);
    }

    public QI2RationalMap getMap() {
	int[] image=M.C.MC.getMap();
	if(image[3]==1) return(DataAux.map());
        if(image[2]==1) return(DataMap.map(image));
	return(DataGauss.map(image));
    }

    /**for ease of notation we copy some routines from other classes*/
    public static QI2RationalFunction times(QI2RationalFunction A,QI2RationalFunction B) {
	return(QI2RationalFunction.times(A,B));
    }

    public static QI2RationalFunction  plus(QI2RationalFunction A,QI2RationalFunction B) {
	return(QI2RationalFunction.plus(A,B));
    }

    public static QI2RationalFunction  minus(QI2RationalFunction A,QI2RationalFunction B) {
	return(QI2RationalFunction.minus(A,B));
    }


    public static QI2Polynomial times(QI2Polynomial A,QI2Polynomial B) {
	return(QI2Polynomial.times(A,B));
    }

    public static QI2Polynomial  plus(QI2Polynomial A,QI2Polynomial B) {
	return(QI2Polynomial.plus(A,B));
    }

    public static QI2Polynomial  minus(QI2Polynomial A,QI2Polynomial B) {
	return(QI2Polynomial.minus(A,B));
    }

    /**and more notation savers*/

    public static QI2RationalFunction  plus(QI2RationalFunction A,QI2 b) {
	return(plus(A,new QI2RationalFunction(b)));

    }
    public static QI2RationalFunction  minus(QI2RationalFunction A,QI2 b) {
	return(minus(A,new QI2RationalFunction(b)));
    }

    public static QI2RationalFunction  minus(QI2RationalFunction A,int n) {
	return(minus(A,new QI2(n,0)));
    }


    public static QI2Polynomial times(QI2Polynomial A,QI2 b) {
	QI2Polynomial B=new QI2Polynomial(b);
	return(times(A,B));
    }

    public static QI2RationalFunction times(QI2RationalFunction A,QI2 b) {
	QI2RationalFunction B=new QI2RationalFunction(b);
	return(times(A,B));
    }


    /**This returns the special quadratic polynomial that
       vanishes on the line y=x and the line y=-phi^{-2} x and is
       positive but small in the interior of the fundamental domain*/

    public static QI2Polynomial special() {
	QI2Polynomial P1=new QI2Polynomial(new QI2(1,0),new QI2(-1,0),new QI2(0,0));
	QI2Polynomial P2=new QI2Polynomial(new QI2(2,-1),new QI2(1,0),new QI2(0,0));
	QI2Polynomial P3=times(P1,P2);
	P3=times(P3,new QI2(55,-34));
	return(P3);
    }



}
